'use strict'

const execa = require('execa')
const moment = require('moment')

function substitute(str, o) {
  return str.replace(/\\?\{([^{}]+)\}/g, (match, name) => {
    if (match.charAt(0) === '\\') {
      return match.slice(1)
    }

    return o[name] === undefined ? '' : o[name]
  })
}

module.exports = {
  website: {
    assets: './assets',
    css: ['style.css']
  },

  filters: {
    timeFormat(time, format) {
      return moment(time).format(format || moment.defaultFormat)
    }
  },

  hooks: {
    page(page) {
      const defaults = {
        position: 'bottom',
        createTpl: 'Created by {user} at {timeStamp}',
        modifyTpl: 'Last modified by {user} at {timeStamp}',
        timeStampFormat: 'YYYY-MM-DD HH:mm:ss'
      }
      const bookRoot = this.resolve('')
      const pluginConfig = this.config.get('pluginsConfig')['git-author']
      const options = Object.assign({}, defaults, pluginConfig)

      const {position} = options
      const {createTpl} = options
      const {modifyTpl} = options
      const {timeStampFormat} = options
      let message = ''
      let creationMessage = ''
      let updateUser = ''
      let createUser = ''
      async function getMessage() {
        const r = await execa.shell(`git log -1 --pretty=oneline --abbrev-commit -- "${page.rawPath}"`, {
          cwd: bookRoot
        })
        message = r.stdout
        if (message.indexOf('on behalf of') === -1 || message.indexOf('Update') === -1) {
          updateUser = 'noUpdateUser'
        } else {
          updateUser = message.split(' ').splice(-1).toString()
        }
      }

      async function getCreationMessage() {
        const s = await execa.shell(`git log --reverse --oneline -- "${page.rawPath}" |head -1`, {
          cwd: bookRoot
        })
        creationMessage = s.stdout
        if (creationMessage.indexOf('Create new file on behalf of') === -1) {
          createUser = 'noCreateUser'
        } else {
          createUser = creationMessage.split(' ').splice(-1).toString()
        }
      }

      getMessage()
      getCreationMessage()

      return execa
        .shell(`git log --format="%an|%at" -- "${page.rawPath}"`, {
          cwd: bookRoot
        })
        .then(ret => {
          // None commit to this file
          if (!ret.stdout) {
            this.log.debug(`none commit to file ${page.path}`)
            return page
          }

          const commits = ret.stdout.split(/\r?\n/).map(log => {
            const arr = log.split('|')
            const timeStamp = moment(arr[1] * 1000).format(timeStampFormat)

            return {
              user: arr[0],
              timestamp: timeStamp,
              timeStamp
            }
          })

          const lastCommit = commits[0]
          const firstCommit = commits.slice(-1)[0]
          if (updateUser !== 'noUpdateUser') {
            lastCommit.user = updateUser
          }

          if (createUser !== 'noCreateUser') {
            firstCommit.user = createUser
          }

          let gitAuthorContent = `<div class="git-author-container git-author-${position}">`

          if (modifyTpl) {
            const modifyMsg = substitute(modifyTpl, lastCommit)
            gitAuthorContent += `<div class="modified">${modifyMsg}</div>`
          }

          if (createTpl) {
            const createMsg = substitute(createTpl, firstCommit)
            gitAuthorContent += `<div class="created">${createMsg}</div>`
          }

          gitAuthorContent += '</div>'

          if (position === 'top') {
            page.content = gitAuthorContent + page.content
          } else {
            page.content += gitAuthorContent
          }

          return page
        })
        .catch(error => {
          this.log.warn('initialize git repository and commit files firstly')
          this.log.warn(error)
          return page
        })
    }
  }
}
